@@ -19,38 +19,23 @@ logger = logging.getLogger(__name__)
 class ButtonFinder:
     def __init__(
         self,
-        row_dist: float = 126.3,
-        col_dist: float = 233.2,
+        row_dist: float = 375 / 3.22,
+        col_dist: float = 655 / 3.22,
         min_button_radius: int = 4,
         max_button_radius: int = 15,
         cluster_penalty: float = 10,
-        region_length: int = 61,
+        roi_length: int = 61,
+        progress_bar: bool = False,
     ):
         self.row_dist = row_dist
         self.col_dist = col_dist
         self.min_button_radius = min_button_radius
         self.max_button_radius = max_button_radius
         self.cluster_penalty = cluster_penalty
-        self.region_length = region_length
-
-    def __call__(
-        self,
-        assay: xr.Dataset,
-        row_dist: float = 126.3,
-        col_dist: float = 233.2,
-        min_button_radius: int = 4,
-        max_button_radius: int = 15,
-        cluster_penalty: float = 10,
-        region_length: int = 61,
-        progress_bar: bool = False,
-    ) -> xr.Dataset:
-        self.row_dist = row_dist
-        self.col_dist = col_dist
-        self.min_button_radius = min_button_radius
-        self.max_button_radius = max_button_radius
-        self.cluster_penalty = cluster_penalty
-        self.region_length = region_length
+        self.roi_length = roi_length
+        self.progress_bar = progress_bar
 
+    def __call__(self, assay: xr.Dataset) -> xr.Dataset:
         num_rows, num_cols = assay.id.shape
         if isinstance(assay.search_channel, str):
             if assay.search_channel in assay.channel:
@@ -68,14 +53,14 @@ class ButtonFinder:
             min_button_dist -= 1
 
         # Create the array of subimage regions.
-        regions = da.empty(
+        roi = da.empty(
             (
                 num_rows,
                 num_cols,
                 assay.dims["channel"],
                 assay.dims["time"],
-                self.region_length,
-                self.region_length,
+                self.roi_length,
+                self.roi_length,
             ),
             dtype=assay.image.dtype,
             chunks=(
@@ -83,44 +68,44 @@ class ButtonFinder:
                 1,
                 assay.dims["channel"],
                 assay.dims["time"],
-                self.region_length,
-                self.region_length,
+                self.roi_length,
+                self.roi_length,
             ),
         )
         assay = assay.assign(
-            region=(
-                ("marker_row", "marker_col", "channel", "time", "row", "col"),
-                regions,
+            roi=(
+                ("marker_row", "marker_col", "channel", "time", "roi_y", "roi_x"),
+                roi,
             ),
             fg=(
-                ("marker_row", "marker_col", "channel", "time", "row", "col"),
+                ("marker_row", "marker_col", "channel", "time", "roi_y", "roi_x"),
                 da.empty_like(
-                    regions,
+                    roi,
                     dtype=bool,
                 ),
             ),
             bg=(
-                ("marker_row", "marker_col", "channel", "time", "row", "col"),
+                ("marker_row", "marker_col", "channel", "time", "roi_y", "roi_x"),
                 da.empty_like(
-                    regions,
+                    roi,
                     dtype=bool,
                 ),
             ),
         )
         # Create the x and y coordinates arrays for each button.
-        assay = assay.assign_coords(
+        assay = assay.assign(
             x=(
-                ["marker_row", "marker_col", "time"],
+                ("marker_row", "marker_col", "time"),
                 np.empty((num_rows, num_cols, assay.dims["time"])),
             ),
             y=(
-                ["marker_row", "marker_col", "time"],
+                ("marker_row", "marker_col", "time"),
                 np.empty((num_rows, num_cols, assay.dims["time"])),
             ),
         )
 
         # Run the button finding algorithm for each timestep.
-        for t, time in enumerate(tqdm.tqdm(assay.time, disable=not progress_bar)):
+        for t, time in enumerate(tqdm.tqdm(assay.time, disable=not self.progress_bar)):
             # Preload all images for this timnepoint so we only read from disk once.
             images = assay.image.sel(time=time).to_numpy()
             points = np.empty((0, 2))
@@ -218,26 +203,26 @@ class ButtonFinder:
 
             # Step 6: Extract a region around each button.
             offsets = np.empty((num_rows, num_cols, 2), dtype=int)
-            regions = np.empty(
-                (num_rows, num_cols, assay.dims["channel"], self.region_length, self.region_length),
-                dtype=assay.region.dtype,
+            roi = np.empty(
+                (num_rows, num_cols, assay.dims["channel"], self.roi_length, self.roi_length),
+                dtype=assay.roi.dtype,
             )
             for i in range(num_rows):
                 for j in range(num_cols):
                     top, bottom, left, right = utils.bounding_box(
                         round(float(assay.x[i, j, t])),
                         round(float(assay.y[i, j, t])),
-                        self.region_length,
-                        assay.dims["im_row"],
-                        assay.dims["im_col"],
+                        self.roi_length,
+                        assay.sizes["im_y"],
+                        assay.sizes["im_x"],
                     )
-                    regions[i, j] = images[:, top:bottom, left:right]
+                    roi[i, j] = images[:, top:bottom, left:right]
                     offsets[i, j] = [left, top]
-            assay.region[:, :, :, t] = regions
+            assay.roi[:, :, :, t] = roi
 
             # Step 7: Compute the foreground and background masks for all buttons.
             fg = np.empty(
-                (num_rows, num_cols, assay.dims["channel"], self.region_length, self.region_length),
+                (num_rows, num_cols, assay.dims["channel"], self.roi_length, self.roi_length),
                 dtype=bool,
             )
             bg = np.empty_like(fg)
@@ -245,27 +230,31 @@ class ButtonFinder:
                 for j in range(num_cols):
                     # TODO: This step should occur over multiple channels.
                     c = np.where(assay.channel == search_channels[0])[0][0]
-                    subimage = utils.to_uint8(regions[i, j, c])
-                    # Filter the subimage to smooth edges and remove noise.
-                    filtered = cv.bilateralFilter(
-                        subimage,
-                        d=9,
-                        sigmaColor=75,
-                        sigmaSpace=75,
-                        borderType=cv.BORDER_DEFAULT,
-                    )
+                    subimage = utils.to_uint8(roi[i, j, c])
 
-                    # Find any circles in the subimage.
-                    circles = cv.HoughCircles(
-                        filtered,
-                        method=cv.HOUGH_GRADIENT,
-                        dp=1,
-                        minDist=50,
-                        param1=20,
-                        param2=5,
-                        minRadius=self.min_button_radius,
-                        maxRadius=self.max_button_radius,
-                    )
+                    # Find circles to refine our button estimate unless we have a blank chamber.
+                    circles = None
+                    if assay.id[i, j] != "":
+                        # Filter the subimage to smooth edges and remove noise.
+                        filtered = cv.bilateralFilter(
+                            subimage,
+                            d=9,
+                            sigmaColor=75,
+                            sigmaSpace=75,
+                            borderType=cv.BORDER_DEFAULT,
+                        )
+
+                        # Find any circles in the subimage.
+                        circles = cv.HoughCircles(
+                            filtered,
+                            method=cv.HOUGH_GRADIENT,
+                            dp=1,
+                            minDist=50,
+                            param1=20,
+                            param2=5,
+                            minRadius=self.min_button_radius,
+                            maxRadius=self.max_button_radius,
+                        )
 
                     # Update our estimate of the button position if we found some circles.
                     if circles is not None:
@@ -280,7 +269,7 @@ class ButtonFinder:
 
                     # Set the foreground (the button) to be a circle of fixed radius.
                     fg_mask = utils.circle(
-                        self.region_length,
+                        self.roi_length,
                         row=y_rel,
                         col=x_rel,
                         radius=self.max_button_radius,
@@ -289,7 +278,7 @@ class ButtonFinder:
 
                     # Set the background to be the annulus around our foreground.
                     bg_mask = utils.circle(
-                        self.region_length,
+                        self.roi_length,
                         row=y_rel,
                         col=x_rel,
                         radius=2 * self.max_button_radius,
@@ -321,11 +310,31 @@ class ButtonFinder:
             assay.fg[:, :, :, t] = fg
             assay.bg[:, :, :, t] = bg
 
+        assay = assay.stack(marker=("marker_row", "marker_col"), create_index=True).transpose(
+            "marker", ...
+        )
+        assay = assay.set_xindex("id")
+
         return assay
 
     @registry.components.register("find_buttons")
-    def make():
-        return ButtonFinder()
+    def make(
+        row_dist: float = 375 / 3.22,
+        col_dist: float = 655 / 3.22,
+        min_button_radius: int = 4,
+        max_button_radius: int = 15,
+        cluster_penalty: float = 10,
+        roi_length: int = 61,
+        progress_bar: bool = False,
+    ):
+        return ButtonFinder(
+            row_dist=row_dist,
+            col_dist=col_dist,
+            min_button_radius=min_button_radius,
+            cluster_penalty=cluster_penalty,
+            roi_length=roi_length,
+            progress_bar=progress_bar,
+        )
 
 
 class BeadFinder:
@@ -333,30 +342,17 @@ class BeadFinder:
         self,
         min_bead_radius: int = 10,
         max_bead_radius: int = 30,
-        region_length: int = 61,
+        roi_length: int = 61,
         param1: int = 50,
         param2: int = 30,
     ):
         self.min_bead_radius = min_bead_radius
         self.max_bead_radius = max_bead_radius
-        self.region_length = region_length
+        self.roi_length = roi_length
         self.param1 = param1
         self.param2 = param2
 
-    def __call__(
-        self,
-        assay: xr.Dataset,
-        min_bead_radius: int = 10,
-        max_bead_radius: int = 30,
-        region_length: int = 61,
-        param1: int = 50,
-        param2: int = 30,
-    ) -> xr.Dataset:
-        self.min_bead_radius = min_bead_radius
-        self.max_bead_radius = max_bead_radius
-        self.region_length = region_length
-        self.param1 = param1
-        self.param2 = param2
+    def __call__(self, assay: xr.Dataset) -> xr.Dataset:
         if isinstance(assay.search_channel, str):
             if assay.search_channel in assay.channel:
                 search_channels = [assay.search_channel]
@@ -413,47 +409,47 @@ class BeadFinder:
             num_beads = len(centers)
             # Create the array of subimage regions.
             assay = assay.assign(
-                region=(
-                    ("marker", "channel", "time", "row", "col"),
+                roi=(
+                    ("marker", "channel", "time", "roi_y", "roi_x"),
                     np.empty(
                         (
                             num_beads,
                             assay.dims["channel"],
                             assay.dims["time"],
-                            self.region_length,
-                            self.region_length,
+                            self.roi_length,
+                            self.roi_length,
                         ),
                         dtype=assay.image.dtype,
                     ),
                 ),
                 fg=(
-                    ("marker", "channel", "time", "row", "col"),
+                    ("marker", "channel", "time", "roi_y", "roi_x"),
                     np.empty(
                         (
                             num_beads,
                             assay.dims["channel"],
                             assay.dims["time"],
-                            self.region_length,
-                            self.region_length,
+                            self.roi_length,
+                            self.roi_length,
                         ),
                         dtype=bool,
                     ),
                 ),
                 bg=(
-                    ("marker", "channel", "time", "row", "col"),
+                    ("marker", "channel", "time", "roi_y", "roi_x"),
                     np.empty(
                         (
                             num_beads,
                             assay.dims["channel"],
                             assay.dims["time"],
-                            self.region_length,
-                            self.region_length,
+                            self.roi_length,
+                            self.roi_length,
                         ),
                         dtype=bool,
                     ),
                 ),
             )
-            assay = assay.assign_coords(
+            assay = assay.assign(
                 x=(
                     ["marker", "time"],
                     np.repeat(centers[:, np.newaxis, 0], assay.dims["time"], axis=1),
@@ -464,23 +460,25 @@ class BeadFinder:
                 ),
             )
 
+            rois = np.empty(assay.roi.shape, dtype=assay.roi.dtype)
+            fgs = np.empty(assay.roi.shape, dtype=bool)
+            bgs = np.empty(assay.roi.shape, dtype=bool)
+            image = assay.image.to_numpy()
             # Compute the foreground and background masks for all buttons.
             for i in range(num_beads):
                 # Set the subimage region for this bead.
                 top, bottom, left, right = utils.bounding_box(
                     round(float(assay.x[i, 0])),
                     round(float(assay.y[i, 0])),
-                    self.region_length,
+                    self.roi_length,
                     image.shape[-2],
                     image.shape[-1],
                 )
-                assay.region[i] = assay.image.sel(
-                    im_row=slice(top, bottom), im_col=slice(left, right)
-                )
+                rois[i] = image[..., top:bottom, left:right]
 
                 # Set the foreground of the bead to be the circle we found.
                 fg_mask = utils.circle(
-                    self.region_length,
+                    self.roi_length,
                     row=round(float(assay.y[i, 0] - top)),
                     col=round(float(assay.x[i, 0] - left)),
                     radius=round(radii[i]),
@@ -492,14 +490,29 @@ class BeadFinder:
                 bg_mask = ~fg_mask
 
                 # Set the masked arrays with our computed masks.
-                assay.fg[i] = fg_mask
-                assay.bg[i] = bg_mask
+                fgs[i] = fg_mask
+                bgs[i] = bg_mask
 
+        assay.fg[:] = fgs
+        assay.bg[:] = bgs
+        assay.roi[:] = rois
         return assay
 
     @registry.components.register("find_beads")
-    def make():
-        return BeadFinder()
+    def make(
+        min_bead_radius: int = 10,
+        max_bead_radius: int = 30,
+        roi_length: int = 61,
+        param1: int = 50,
+        param2: int = 30,
+    ):
+        return BeadFinder(
+            min_bead_radius=min_bead_radius,
+            max_bead_radius=max_bead_radius,
+            roi_length=roi_length,
+            param1=param1,
+            param2=param2,
+        )
 
 
 def cluster_1d(
