@@ -1,6 +1,7 @@
 from __future__ import annotations
 from typing import Callable
 
+import confection
 from numpy.typing import ArrayLike
 import xarray as xr
 
@@ -9,8 +10,9 @@ import magnify.utils as utils
 
 
 class Pipeline:
-    def __init__(self, reader: str):
+    def __init__(self, reader: str, config: dict[str, dict[str, str]]):
         self.reader: Callable[[ArrayLike | str], xr.Dataset] = registry.readers.get(reader)()
+        self.config = confection.Config(config)
         self.components: list[Callable[[xr.Dataset], xr.Dataset]] = []
 
     def __call__(
@@ -20,7 +22,6 @@ class Pipeline:
         search_on: str = "egfp",
         times: Sequence[int] | None = None,
         channels: Sequence[str] | None = None,
-        **kwargs,
     ) -> xr.Dataset | list[xr.Dataset]:
         inputs = self.reader(
             data=data, names=names, search_on=search_on, times=times, channels=channels
@@ -28,7 +29,7 @@ class Pipeline:
         assays = []
         for assay in inputs:
             for name, component in self.components:
-                assay = component(assay, **utils.valid_kwargs(kwargs, component))
+                assay = component(assay)
 
             assays.append(assay)
 
@@ -38,4 +39,6 @@ class Pipeline:
         return assays
 
     def add_pipe(self, name: str) -> None:
-        self.components.append((name, registry.components.get(name)()))
+        component_factory = registry.components.get(name)
+        component = component_factory(**utils.valid_kwargs(self.config, component_factory))
+        self.components.append((name, component))
