@@ -2,6 +2,7 @@ from __future__ import annotations
 from collections.abc import Callable
 from typing import Any
 import inspect
+import re
 
 import cv2 as cv
 import numpy as np
@@ -25,18 +26,18 @@ def ceildiv(a: int, b: int) -> int:
 
 
 def bounding_box(
-    row: int, col: int, box_length: int, image_width: int, image_height: int
+    x: int, y: int, box_length: int, image_width: int, image_height: int
 ) -> tuple[int, int, int, int]:
-    top = row - box_length // 2
-    bottom = row + ceildiv(box_length, 2)
+    top = y - box_length // 2
+    bottom = y + ceildiv(box_length, 2)
     if top < 0:
         bottom += -top
         top = 0
     if bottom > image_height:
         top -= bottom - image_height
         bottom = image_height
-    left = col - box_length // 2
-    right = col + ceildiv(box_length, 2)
+    left = x - box_length // 2
+    right = x + ceildiv(box_length, 2)
     if left < 0:
         right += -left
         left = 0
@@ -49,3 +50,8 @@ def bounding_box(
 def valid_kwargs(kwargs: dict[str, Any], func: Callable) -> dict[str, Any]:
     args = list(inspect.signature(func).parameters)
     return {k: kwargs[k] for k in kwargs if k in args}
+
+
+def natural_sort_key(s: str) -> list[str]:
+    reg = re.compile("([0-9]+)")
+    return [int(text) if text.isdigit() else text.lower() for text in reg.split(s)]
